using System;
using System.IO;
using System.Collections.Specialized;
using System.Web.Services.Protocols;

namespace Server_Proxy
{
	///////////////////////////////////////////////////////////////////////
	//	Enumeration of attachment type.
	///////////////////////////////////////////////////////////////////////

	public enum AttachmentType
	{
		DownloadFile,
		GetContent,
		SelectPage,
		GetTOC,
		GetStyleSheet,
		GetStaticData,
		GetDynamicData,
		GetEmbededComponent,
		UploadFile
	}

	///////////////////////////////////////////////////////////////////////
	//	UploadExtension Attribute
	///////////////////////////////////////////////////////////////////////

	[AttributeUsage(AttributeTargets.Method)]
	public class UploadExtensionAttribute : SoapExtensionAttribute
	{
		private int priority;
		private AttachmentType m_type;

		public override Type ExtensionType
		{
			get { return typeof(UploadExtension); }
		}

		public override int Priority 
		{
			get { return priority; }
			set { priority = value; }
		}
        
		public AttachmentType Type
		{
			get
			{
				return m_type;
			}
			set
			{
				m_type = value;
			}
		}
	}

	///////////////////////////////////////////////////////////////////////////
	//  UploadExtension class:
	//      Derived from soap extension class
	///////////////////////////////////////////////////////////////////////////

	public class UploadExtension : SoapExtension 
	{
		private const int BufferSize = 5000;
		private const string Boundary = "--MIME_boundary";
		private const string Termination = "--MIME_boundary--";

		private MemoryStream m_newStream;
		private Stream m_oldStream;
		private MemoryStream m_bufferStream;

		private bool m_isSerialize = true;

		private AttachmentType m_type;

		public override object GetInitializer(LogicalMethodInfo methodInfo, SoapExtensionAttribute attribute) 
		{
			return ((UploadExtensionAttribute)attribute).Type;
		}

		public override object GetInitializer(Type type) 
		{
			return (Type) type;
		}
        
		public override void Initialize(object initializer) 
		{
			m_type = (AttachmentType) initializer;
		}

		///////////////////////////////////////////////////////////////////////
		//	This is to allow you to access the stream buffer
		///////////////////////////////////////////////////////////////////////

		public override Stream ChainStream( Stream stream )
		{
			if(m_isSerialize)
			{
				m_isSerialize = false;
				m_oldStream = stream;
				m_newStream = new MemoryStream();
				return m_newStream;
			}
			else
			{
				m_isSerialize = true;
				return stream;
			}
		}
		
		///////////////////////////////////////////////////////////////////////
		//	Process the outbound message
		///////////////////////////////////////////////////////////////////////

		public override void ProcessMessage(SoapMessage message) 
		{
			switch (message.Stage) 
			{
				case SoapMessageStage.BeforeSerialize:
				{
					m_bufferStream = new MemoryStream();

					//	Process the stream before the serialization.
					MemoryStream l_buf = new MemoryStream();
					StreamWriter l_bufWriter = new StreamWriter(l_buf);
					l_bufWriter.WriteLine();
					l_bufWriter.WriteLine(Boundary);
					l_bufWriter.WriteLine("Content-Type: text/xml; charset=\"utf-8\"");
					l_bufWriter.WriteLine("Content-Transfer-Encoding: 8bit");
					l_bufWriter.WriteLine("Content-ID: <request.xml>");
					l_bufWriter.Flush();

					l_buf.Position = 0;
					l_buf.WriteTo(m_bufferStream);

					message.ContentType = "multipart/related;boundary=\"MIME_boundary\";type=\"text/xml\";start=\"<request.xml>\"";					

					break;
				}
				case SoapMessageStage.AfterSerialize:
				{
					MemoryStream l_tmpBuf = new MemoryStream();
					StreamWriter l_tmpBufWriter = new StreamWriter(l_tmpBuf);
					l_tmpBufWriter.WriteLine("Content-Length: " + message.Stream.Length);
					l_tmpBufWriter.WriteLine();
					l_tmpBufWriter.Flush();

					l_tmpBuf.Position = 0;
					l_tmpBuf.WriteTo(m_bufferStream);

					message.Stream.Position = 0;
					((MemoryStream) message.Stream).WriteTo(m_bufferStream);

					//	Construct the mime section
					MemoryStream l_buf = new MemoryStream();
					StreamWriter l_bufWriter = new StreamWriter(l_buf);
					l_bufWriter.WriteLine();
					l_bufWriter.WriteLine();
					l_bufWriter.WriteLine(Boundary);
					l_bufWriter.WriteLine("Content-Type: binary");
					l_bufWriter.WriteLine("Content-Transfer-Encoding: 8bit");
					l_bufWriter.WriteLine("Content-ID: Attachment");
					l_bufWriter.WriteLine();
					l_bufWriter.Flush();

					//	Attach the uploaded file
					BinaryReader l_reader = new BinaryReader(ActuateAPIEx.UploadStream);
					BinaryWriter l_writer = new BinaryWriter(l_buf);
					byte[] l_bytes = new byte[BufferSize];
					int l_count = 0;
					while(true)
					{
						l_count = l_reader.Read(l_bytes, 0, BufferSize);
						if(l_count == 0)
						{
							break;
						}
						l_writer.Write(l_bytes, 0, l_count);
					}
				
					l_buf.Position = 0;
					l_buf.WriteTo(m_bufferStream);

					//	Prepare the termination section
					MemoryStream l_terminate = new MemoryStream();
					StreamWriter l_terminateStream = new StreamWriter(l_terminate);
					l_terminateStream.WriteLine();
					l_terminateStream.WriteLine(Termination);
					l_terminateStream.Flush();
					l_terminate.Position = 0;
					l_terminate.WriteTo(m_bufferStream);

					//	Need to dump the stream back into the system's stream.
					m_bufferStream.Position = 0;
					m_bufferStream.WriteTo(m_oldStream);

					break;
				}
				case SoapMessageStage.AfterDeserialize:
				case SoapMessageStage.BeforeDeserialize:
				{
					break;
				}
			}
		}
	}

	///////////////////////////////////////////////////////////////////////////
	//  DownloadExtensionAttribute class
	//      Attribute for TraceExtension class
	///////////////////////////////////////////////////////////////////////////

	[AttributeUsage(AttributeTargets.Method)]
	public class DownloadExtensionAttribute : SoapExtensionAttribute
	{
		private int priority;
		private AttachmentType m_type;

		public override Type ExtensionType
		{
			get { return typeof(DownloadExtension); }
		}

		public override int Priority 
		{
			get { return priority; }
			set { priority = value; }
		}
        
		//	Type parameter identify the API type
		public AttachmentType Type
		{
			get
			{
				return m_type;
			}
			set
			{
				m_type = value;
			}
		}
	}

	///////////////////////////////////////////////////////////////////////////
	//  DownloadExtension class:
	//
	//      Derived from soap extension class
	//
	///////////////////////////////////////////////////////////////////////////

	public class DownloadExtension : SoapExtension 
	{
		private const int BufferSize = 1000000;
		private string Boundary = "--Mime_boundary\r\n";
		private string Termination = "--Mime_boundary--\r\n";

		private bool m_isSerialize = true;

		private AttachmentType m_type;

		private MemoryStream m_newStream;
		private Stream m_oldStream;

		public override object GetInitializer(Type type) 
		{
			return (Type) type;
		}

		public override object GetInitializer(LogicalMethodInfo methodInfo, SoapExtensionAttribute attribute) 
		{
			return ((DownloadExtensionAttribute)attribute).Type;
		}
        
		public override void Initialize(object initializer) 
		{
			m_type = (AttachmentType) initializer;
		}

		///////////////////////////////////////////////////////////////////////
		//	This is to allow you to access the stream buffer
		///////////////////////////////////////////////////////////////////////

		public override Stream ChainStream( Stream stream )
		{
			if(!m_isSerialize)
			{
				m_isSerialize = true;
				m_oldStream = stream;
				m_newStream = new MemoryStream();
				return m_newStream;
			}
			else
			{
				m_isSerialize = false;
				return stream;
			}
		}

		///////////////////////////////////////////////////////////////////////
		//	Process the inbound message in this function
		///////////////////////////////////////////////////////////////////////

		public override void ProcessMessage(SoapMessage message) 
		{
			switch (message.Stage) 
			{
				case SoapMessageStage.BeforeSerialize:
				case SoapMessageStage.AfterSerialize:
				{
					break;
				}
				case SoapMessageStage.BeforeDeserialize:
				{
					// for reading entire SOAP message with MIME attachments
					BinaryReader reader = new BinaryReader(m_oldStream);
					// for writing SOAP message
					BinaryWriter soapWriter = new BinaryWriter(m_newStream);
					// for writing file attachment
					string tempFile = Path.GetTempFileName();
					FileStream fileStream = new FileStream(tempFile, FileMode.Create);

					NameValueCollection mimeHeaders;
					byte[] buf, buf2;

					//  read the SOAP response message
					int attachmentCount = 0;
					do
					{
						GetSingleLineInByte(reader, out buf);

						if(Compare(buf, Boundary))
						{
							//  need to skip two mime boundaries
							if(attachmentCount > 0)
							{
								break;
							}
							else
							{
								attachmentCount++;
								GetMimeHeaders(reader);
								continue;
							}
						}

						if(attachmentCount > 0)
						{
							soapWriter.Write(buf);
						}
					} while (true);

					m_newStream.Position = 0;
					mimeHeaders = GetMimeHeaders(reader);

					// read the attachment
					buf = buf2 = null;
					do
					{
						buf2 = reader.ReadBytes(BufferSize);

						int terminationStartIndex = FindSubstring(buf, buf2, Termination);

						// write buffer contents to the output file
						if(terminationStartIndex >= 0)
						{
							fileStream.Write(buf, 0, terminationStartIndex);
							break;
						}
						else
						{
							if(buf != null)
							{
								fileStream.Write(buf, 0, buf.Length);
							}

							if(buf2.Length > Termination.Length)
							{
								// write buffer contents with the exception of last Termination.Length
								// bytes in case if the last MIME boundary is split across buffers
								fileStream.Write(buf2, 0, buf2.Length - Termination.Length);
								ShiftBuffer(ref buf2, buf2.Length - Termination.Length);
							}
						}

						buf = buf2;

					} while(true);

					fileStream.Flush();
					fileStream.Close();

					// save file location in soap header, to retrieve it in wrapper class
					message.Headers.Add(new DownloadFileHeader(tempFile));

					// this is necessary
					message.ContentType = "text/xml;charset=\"utf-8\"";
					break;
				}
				case SoapMessageStage.AfterDeserialize:
				{
					break;
				}
			}
		}

		///////////////////////////////////////////////////////////////////////
		// Get a single line from the inbound stream as a byte array
		///////////////////////////////////////////////////////////////////////

		private int GetSingleLineInByte(BinaryReader reader, out byte[] buffer)
		{
			MemoryStream l_stream = new MemoryStream();
			BinaryWriter l_writer = new BinaryWriter(l_stream);

			byte l_char, l_prevChar = 0x0;

			do
			{
				try
				{
					l_char = reader.ReadByte();
				}
				catch(EndOfStreamException)
				{
					break;
				}
				catch(ObjectDisposedException)
				{
					break;
				}
				catch(IOException)
				{
					break;
				}
 
				l_writer.Write(l_char);

				if(l_char == 0x0A && l_prevChar == 0x0D)
				{
					break;
				}

				l_prevChar = l_char;

			} while(true);

			l_stream.Seek(0, SeekOrigin.Begin);
			BinaryReader l_reader = new BinaryReader(l_stream);

			buffer = l_reader.ReadBytes((int) l_stream.Length);

			return (int) l_stream.Length;
		}
        
		///////////////////////////////////////////////////////////////////////
		// Get a single line from the inbound stream as a char array
		///////////////////////////////////////////////////////////////////////

		private int GetSingleLineInChar(BinaryReader reader, out char[] line)
		{
			MemoryStream l_stream = new MemoryStream();
			BinaryWriter l_writer = new BinaryWriter(l_stream);

			do
			{
				char l_char;
				try
				{
					l_char = reader.ReadChar();
				}
				catch(EndOfStreamException)
				{
					break;
				}
				catch(ObjectDisposedException)
				{
					break;
				}
				catch(IOException)
				{
					break;
				}
 
				l_writer.Write(l_char);

				if(l_char == (char) 10)
				{
					break;
				}

			} while(true);

			l_stream.Seek(0, SeekOrigin.Begin);
			BinaryReader l_reader = new BinaryReader(l_stream);

			line = l_reader.ReadChars((int) l_stream.Length);

			return (int) l_stream.Length;
		}

		///////////////////////////////////////////////////////////////////////
		// Compire byte array to a string
		///////////////////////////////////////////////////////////////////////

		private bool Compare(byte[] s1, string s2)
		{
			if(s1.Length != s2.Length)
			{
				return false;
			}

			for(int i = 0; i < s1.Length; i++)
			{
				if(s1[i] != (byte)s2[i])
				{
					return false;
				}
			}

			return true;
		}

		///////////////////////////////////////////////////////////////////////
		// Get a single line from the inbound stream
		// The method returns position in buf1 where the str starts (an that
		// can possibly spread across the two buffers)
		// returns -1 otherwise
		///////////////////////////////////////////////////////////////////////

		private int FindSubstring(byte[] buf1, byte[] buf2, string str)
		{
			if(buf1 == null || buf2 == null || str == null || str.Length > buf1.Length + buf2.Length || str.Length == 0)
			{
				return -1;
			}

			// look for str in the first buffer
			int j = 0, begPos = 0;
			for(int i = 0; i < buf1.Length; i++)
			{
				if(buf1[i] == (byte)str[j])
				{
					if(j == 0)
					{
						begPos = i;
					}

					j++;

					if(j >= str.Length)
					{
						return begPos;
					}
				}
				else
				{
					j = 0;
				}
			}

			// if only a portion of str is found in the first buffer
			// then continue to the second buffer
			if(j > 0)
			{
				for(int i = 0; i < str.Length - j; i++)
				{
					if(buf2[i] == (byte)str[j])
					{
						j++;
					}
					else
					{
						return -1;
					}

					if(j >= str.Length)
					{
						return begPos;
					}
				}
			}

			return -1;
		}

		///////////////////////////////////////////////////////////////////////
		// Shift the byte array comtents to left starting at startIndex
		///////////////////////////////////////////////////////////////////////

		private int ShiftBuffer(ref byte[] buffer, int startIndex)
		{
			if(buffer == null || buffer.Length <= startIndex)
			{
				return -1;
			}

			byte[] l_buf = new byte[buffer.Length - startIndex];

			for(int i = 0; i < l_buf.Length; i++)
			{
				l_buf[i] = buffer[startIndex + i];
			}

			buffer = l_buf;
			l_buf = null;

			return buffer.Length;
		}
        
		///////////////////////////////////////////////////////////////////////
		//	Get the mime header from the stream.
		//	This is a workaround since mime header contains the same
		//	information as those in the saop envelopment.
		///////////////////////////////////////////////////////////////////////

		private NameValueCollection GetMimeHeaders(BinaryReader reader)
		{
			NameValueCollection l_collection = new NameValueCollection();

			for(int i = 0; i < 4; i++)
			{
				char[] l_line;
				GetSingleLineInChar(reader, out l_line);
				string l_string = new string(l_line);
				if(l_string != null)
				{
					string[] l_array = l_string.Trim().Split(':');
					if(l_array.Length == 2)
					{
						if(l_array[0] != null && l_array[1] != null)
						{
							l_collection.Add(l_array[0], l_array[1]);
						}
					}
				}
			}
           
			return l_collection;
		}

	}

	[System.Xml.Serialization.XmlTypeAttribute(Namespace = "http://schemas.actuate.com/actuate11")]
	[System.Xml.Serialization.XmlRootAttribute(Namespace = "http://schemas.actuate.com/actuate11", IsNullable = false)]
	public class DownloadFileHeader : SoapHeader
	{
		public DownloadFileHeader() { }

		public DownloadFileHeader(string fileName)
		{
			TargetFileName = fileName;
		}

		public string TargetFileName;
	}

}